@@ -82,7 +82,7 @@ NNS.dep = function(x,
     # No dependence if only a single value
     if(length(unique(x))==1 | length(unique(y))==1){
       if(print.map){
-        NNS.part(x, y, order=1, Voronoi = TRUE)
+        NNS.part(x, y, order = 1, Voronoi = TRUE, noise.reduction = "median")
       }
 
       return(list("Correlation" = 0,
@@ -146,7 +146,7 @@ NNS.dep = function(x,
 
 
     if(l >= 150 & print.map){
-      NNS.part(x, y, order = order, Voronoi = TRUE)
+      NNS.part(x, y, order = order, min.obs.stop = TRUE, Voronoi = TRUE, noise.reduction = "median")
     }
 
     return(list("Correlation" = mean(unlist(lapply(nns.dep, `[[`, 1))),
